@@ -14,6 +14,12 @@ import hypervisor
 PROFILE_VERSION = 0
 
 SYS_CLASS_DMI = "/sys/class/dmi/id"
+SECRET_ID_FILE = "/etc/fireinfo-id"
+
+INVALID_ID_STRINGS = (
+	"OEM", "O.E.M.", "o.e.m.",
+	"N/A", "n/a",
+)
 
 class Singleton(type):
 	def __init__(cls, name, bases, dict):
@@ -117,9 +123,6 @@ class System(object):
 			"public_id" : self.public_id,
 			"private_id" : self.private_id,
 
-			# XXX just for testing:
-			"secret_id" : self._unique_id,
-
 			# Actual profile data
 			"profile" : p,
 		}
@@ -135,7 +138,7 @@ class System(object):
 			This returns a globally (hopefully) ID to identify the host
 			later (by request) in the database.
 		"""
-		public_id = self._unique_id
+		public_id = self.secret_id
 		if not public_id:
 			return "0" * 40
 
@@ -150,7 +153,7 @@ class System(object):
 			No one could ever guess this without access to the host.
 		"""
 		private_id = ""
-		for i in reversed(self._unique_id):
+		for i in reversed(self.secret_id):
 			private_id += i
 
 		if not private_id:
@@ -158,6 +161,17 @@ class System(object):
 
 		return hashlib.sha1(private_id).hexdigest()
 
+	@property
+	def secret_id(self):
+		"""
+			Read a "secret" ID from a file if available
+			or calculate it from the hardware.
+		"""
+		if os.path.exists(SECRET_ID_FILE):
+			return read_from_file(SECRET_ID_FILE)
+
+		return hashlib.sha1(self._unique_id).hexdigest()
+
 	@property
 	def _unique_id(self):
 		"""
@@ -171,7 +185,7 @@ class System(object):
 		# Virtual machines (for example) and some boards have a UUID
 		# which is globally unique.
 		for file in ("product_uuid", "product_serial", "chassis_serial"):
-			id = read_from_file(os.path.join(SYS_CLASS_DMI, file)) or ""
+			id = read_from_file(os.path.join(SYS_CLASS_DMI, file))
 			ids.append(id)
 
 		# Use serial number from root disk (if available)
@@ -179,9 +193,22 @@ class System(object):
 		if root_disk_serial:
 			ids.append(root_disk_serial)
 
+		# Sort out all bogous or invalid strings from the list.
+		_ids = []
+		for id in ids:
+			for i in INVALID_ID_STRINGS:
+				if i in id:
+					id = None
+					break
+
+			if id:
+				_ids.append(id)
+
+		ids = _ids
+
 		# As last resort, we use the UUID from pakfire.
 		if not ids:
-			id = read_from_file("/opt/pakfire/db/uuid") or ""
+			id = read_from_file("/opt/pakfire/db/uuid")
 			ids.append(id)
 
 		return "#".join(ids)
@@ -258,8 +285,12 @@ class System(object):
 
 	@property
 	def root_disk_serial(self):
-		return _fireinfo.get_harddisk_serial("/dev/%s" % self.root_disk).rstrip()
-					
+		serial = _fireinfo.get_harddisk_serial("/dev/%s" % self.root_disk)
+
+		if serial:
+			# Strip all spaces
+			return serial.strip()
+
 	def scan(self):
 		toscan = (("/sys/bus/pci/devices", device.PCIDevice),
 		("/sys/bus/usb/devices", device.USBDevice))
